package data.scripts.shipsystems;

import java.awt.Color;
import java.util.ArrayList;
import java.util.EnumSet;
import java.util.List;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.combat.MutableShipStatsAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.WeaponAPI.WeaponType;
//import com.fs.starfarer.api.combat.WeaponAPI;
import com.fs.starfarer.api.util.Misc;

import com.fs.starfarer.api.impl.combat.BaseShipSystemScript;
//import com.fs.starfarer.api.combat.ShipEngineControllerAPI;
//import com.fs.starfarer.api.combat.ShipEngineControllerAPI.ShipEngineAPI;

public class dara_PrecisionOpticsStats extends BaseShipSystemScript {
	public static final Object KEY_JITTER = new Object();
	
	public static final float BEAM_BONUS = 50f;
	public static final float ENERGY_BONUS = 10f;
	
	//public static final Color JITTER_UNDER_COLOR = new Color(255,0,0,125);
	//public static final Color JITTER_COLOR = new Color(255,0,0,150);
	public static final Color GUN_GLOW_COLOR = new Color(0,75,255,75);

	@Override
	public void apply(MutableShipStatsAPI stats, String id, State state, float effectLevel) {
		ShipAPI ship = null;
		if (stats.getEntity() instanceof ShipAPI) {
			ship = (ShipAPI) stats.getEntity();
		} else {
			return;
		}
		
		stats.getBeamWeaponDamageMult().modifyPercent(id, BEAM_BONUS * effectLevel);
		stats.getEnergyWeaponDamageMult().modifyPercent(id, ENERGY_BONUS * effectLevel);
		
		if (effectLevel > 0) {
			float jitterLevel = effectLevel;
			float maxRangeBonus = 5f;
			float jitterRangeBonus = jitterLevel * maxRangeBonus;
			for (ShipAPI fighter : getFighters(ship)) {
				if (fighter.isHulk()) continue;
				MutableShipStatsAPI fStats = fighter.getMutableStats();
				
				fStats.getBeamWeaponDamageMult().modifyPercent(id, BEAM_BONUS * effectLevel);
				fStats.getEnergyWeaponDamageMult().modifyPercent(id, ENERGY_BONUS * effectLevel);
				
				if (jitterLevel > 0) {
					//fighter.setWeaponGlow(effectLevel, Misc.setAlpha(JITTER_UNDER_COLOR, 255), EnumSet.allOf(WeaponType.class));
					fighter.setWeaponGlow(effectLevel, Misc.setAlpha(GUN_GLOW_COLOR, 200), EnumSet.of(WeaponType.ENERGY));
					//for (ShipEngineAPI flame : ShipEngineControllerAPI.getShipEngines()) {
					//	flame.extendFlame(fighter, 1f, 1f, 1.5f);
					//}
					
					//fighter.setJitterUnder(KEY_JITTER, JITTER_UNDER_COLOR, jitterLevel, 5, 0f, jitterRangeBonus);
					//fighter.setJitter(KEY_JITTER, JITTER_COLOR, jitterLevel, 2, 0f, 0 + jitterRangeBonus * 1f);
					Global.getSoundPlayer().playLoop("system_targeting_feed_loop", ship, 1f, 1f, fighter.getLocation(), fighter.getVelocity());
				}
			}
		}
	}
	
	private List<ShipAPI> getFighters(ShipAPI carrier) {
		List<ShipAPI> result = new ArrayList<ShipAPI>();
		
		for (ShipAPI ship : Global.getCombatEngine().getShips()) {
			if (!ship.isFighter()) continue;
			if (ship.getWing() == null) continue;
			if (ship.getWing().getSourceShip() == carrier) {
				result.add(ship);
			}
		}
		
		return result;
	}
	
	@Override
	public void unapply(MutableShipStatsAPI stats, String id) {
		ShipAPI ship = null;
		if (stats.getEntity() instanceof ShipAPI) {
			ship = (ShipAPI) stats.getEntity();
		} else {
			return;
		}

		stats.getBeamWeaponDamageMult().unmodify(id);
		stats.getEnergyWeaponDamageMult().unmodify(id);

		for (ShipAPI fighter : getFighters(ship)) {
			if (fighter.isHulk()) continue;
			MutableShipStatsAPI fStats = fighter.getMutableStats();

			fStats.getBeamWeaponDamageMult().unmodify(id);
			fStats.getEnergyWeaponDamageMult().unmodify(id);
		}
	}
	
	@Override
	public StatusData getStatusData(int index, State state, float effectLevel) {
		//float percent = DAMAGE_BONUS * effectLevel;
		if (index == 0) {
			return new StatusData("+" + Misc.getRoundedValueMaxOneAfterDecimal(BEAM_BONUS * effectLevel) + "% ship & fighter beam damage", false);
		}
		if (index == 1) {
			return new StatusData("+" + Misc.getRoundedValueMaxOneAfterDecimal(ENERGY_BONUS * effectLevel) + "% ship & fighter energy damage", false);
		}
		return null;
	}

	
}








