package data.scripts.shipsystems;

import java.awt.Color;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.combat.MutableShipStatsAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.plugins.ShipSystemStatsScript;
import com.fs.starfarer.api.combat.ShipEngineControllerAPI.ShipEngineAPI;
import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.impl.campaign.ids.HullMods;
import com.fs.starfarer.api.impl.combat.BaseShipSystemScript;

public class dara_StormDriveStats extends BaseShipSystemScript {

	public static float SPEED_BONUS = 75f;
	public static float ZFLUX_BONUS = 50f;
	public static float EMP_RESIST = 100f;
        
        public static float SO_SPEED_BONUS = 100f;  // On top of the regular boost, for a total of +175 top speed.
        public static float SO_MALF_THRESHOLD = 0.5f; // Engine malfunctions start above this flux level.
        public static float SO_MALF_CHANCE = 0.08f; // 8% chance per second. For the Lysander's seven engine plumes
                                                    // and ~9 second system duration, should cause ~5 engine failures
                                                    // per usage. In testing it seems that this can never cause a
                                                    // full engine flameout, and an SO build is still viable.
	
	private Color color = new Color(150,150,255,255);
        
        protected Object STATUSKEY1 = new Object(); // Gotta have this for the additional status data to attach to.
	
	public void apply(MutableShipStatsAPI stats, String id, State state, float effectLevel) {
		if (state == ShipSystemStatsScript.State.OUT) {
			stats.getMaxSpeed().unmodify(id); // to slow down ship to its regular top speed while powering drive down
			//stats.getMaxTurnRate().unmodify(id);

		} else {
			//stats.getZeroFluxSpeedBoost().modifyFlat(id, ZFLUX_BONUS);
			stats.getMaxSpeed().modifyFlat(id, SPEED_BONUS);
			stats.getAcceleration().modifyPercent(id, SPEED_BONUS * 3f * effectLevel);
			stats.getDeceleration().modifyMult(id, 0f);	// Cannot decelerate
			stats.getEmpDamageTakenMult().modifyMult(id, (100f - EMP_RESIST));
		}
		
		if (stats.getEntity() instanceof ShipAPI) {
			ShipAPI ship = (ShipAPI) stats.getEntity();
			
			ship.getEngineController().fadeToOtherColor(this, color, new Color(0,0,0,0), effectLevel, 0.67f);
			//ship.getEngineController().fadeToOtherColor(this, Color.white, new Color(0,0,0,0), effectLevel, 0.67f);
			ship.getEngineController().extendFlame(this, 2f * effectLevel, 0f * effectLevel, 0f * effectLevel);

			//ship.setAffectedByNebula(false);  // Redundant as of 0.9
                        if (ship.getVariant().getHullMods().contains("safetyoverrides")) {
                            //stats.getEngineMalfunctionChance().modifyFlat(id, SO_MALF_CHANCE);
                            float FLUX = ship.getFluxTracker().getFluxLevel();	// Get the ship's current flux value (float from 0 to 1)
                            if (FLUX > SO_MALF_THRESHOLD) {
                                stats.getMaxSpeed().modifyFlat(id, (SPEED_BONUS + SO_SPEED_BONUS));
                                stats.getEngineMalfunctionChance().modifyFlat(id, SO_MALF_CHANCE);
                                Global.getCombatEngine().maintainStatusForPlayerShip(STATUSKEY1, "graphics/icons/warning_beacon.png", "stormrider drive - flux handler", "safeties interrupted! alert! alert!", true);
                                // Above 50% flux, if Safety Overrides is installed, the Lysander gets a massive
                                // extra speed boost, but also starts suffering serious engine malfunctions, so it
                                // can normally only take advantage of it for a few moments.
                            }
                        } else {
                            stats.getZeroFluxSpeedBoost().modifyFlat(id, ZFLUX_BONUS);
                        }

		}
	}
	public void unapply(MutableShipStatsAPI stats, String id) {
		stats.getZeroFluxSpeedBoost().unmodify(id);
		stats.getMaxSpeed().unmodify(id);
		stats.getAcceleration().unmodify(id);
		stats.getDeceleration().unmodify(id);
		stats.getEmpDamageTakenMult().unmodify(id);
                stats.getEngineMalfunctionChance().unmodify(id);
		if (stats.getEntity() instanceof ShipAPI) {
			ShipAPI ship = (ShipAPI) stats.getEntity();
			//ship.setAffectedByNebula(true);
                        
                        //if (ship.getVariant().getHullMods().contains("safetyoverrides")) {
                        //    stats.getEngineMalfunctionChance().unmodify(id);
                        //}
		}
	}
	
	public StatusData getStatusData(int index, State state, float effectLevel) {
		if (index == 0) {
			return new StatusData("+" + (int)SPEED_BONUS + " top speed", false);

		} else if (index == 1) {
                        return new StatusData("" + (int)EMP_RESIST + "% less emp damage taken", false);

		} else if (index == 2) {
                //        return new StatusData("movement unhindered by nebula", false);

		//} else if (index == 3) {
			return new StatusData("cannot decelerate", true);
                        
		}
		return null;
	}
}
